package com.webchat.service.listener;

import com.webchat.common.enums.RoleCodeEnum;
import com.webchat.common.util.JsonUtil;
import com.webchat.controller.client.GroupVideoChatDcsWebSocket;
import com.webchat.controller.client.GroupVideoChatWebSocket;
import com.webchat.domain.vo.request.mess.VideoChatMessageRequestVO;
import com.webchat.domain.vo.response.UserBaseResponseInfoVO;
import com.webchat.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.connection.MessageListener;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.websocket.Session;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
@Component
public class RedisGroupVideoOfferReceiveMessageListener implements MessageListener {

    @Autowired
    private UserService userService;

    private final RedisTemplate redisTemplate;

    public RedisGroupVideoOfferReceiveMessageListener(@Qualifier("redisTemplate") RedisTemplate redisTemplate) {
        this.redisTemplate = redisTemplate;
    }

    @Override
    public void onMessage(Message message, byte[] bytes) {
        String channel = (String) redisTemplate.getStringSerializer().deserialize(message.getChannel());
        String messageStr = (String) redisTemplate.getValueSerializer().deserialize(message.getBody());
        log.info("redis message listener ======> channel:{}, message:{}", channel, messageStr);
        this.handleVideoChattingMessage(messageStr);
    }
    /**
     * 发送音视频通话邀请
     *
     * @param message
     */
    private void handleVideoChattingMessage(String message) {
        VideoChatMessageRequestVO chatMessage = JsonUtil.fromJson(message, VideoChatMessageRequestVO.class);
        if (chatMessage == null) {
            return;
        }
        String groupId = chatMessage.getGroupId();
        String targetUserId = chatMessage.getTargetUserId();
        Map<String, GroupVideoChatDcsWebSocket> wsMap = GroupVideoChatDcsWebSocket.clients.get(groupId);
        UserBaseResponseInfoVO receiver = userService.getUserBaseInfoByUserId(targetUserId);
        List<String> receiverUserIds = new ArrayList<>();
        if (RoleCodeEnum.GROUP.getCode().equals(receiver.getRoleCode())) {
            // 群组
            receiverUserIds = userService.getGroupUserIdsFromCache(targetUserId);
        } else {
            // 个人
            receiverUserIds.add(targetUserId);
        }
        receiverUserIds.remove(chatMessage.getUserId());
        if (CollectionUtils.isEmpty(receiverUserIds)) {
            return;
        }
        for (String receiverUserId: receiverUserIds) {
            GroupVideoChatDcsWebSocket ws = wsMap.get(receiverUserId);
            if (ws == null) {
                continue;
            }
            Session session = ws.getSession();
            if (session == null || !session.isOpen()) {
                continue;
            }
            try {
                session.getBasicRemote().sendText(message);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }
}
